#include <FastLED.h>
#include <IRremote.hpp>

// IR Remote
const int IR_RECEIVE_PIN  = 9;

// WS2812 LED Strip
#define LED1_PIN     7
#define LED2_PIN     8
#define NUM_LEDS    12
CRGB leds1[NUM_LEDS];
CRGB leds2[NUM_LEDS];

// Color
int start = 0;
int rgbArray[NUM_LEDS][3] = {
                              {255, 0, 0}, {255, 127, 0}, {255, 255, 0}, {127, 255, 0},
                              {0, 255, 0}, {0, 255, 127}, {0, 255, 255}, {0, 127, 255},
                              {0, 0, 255}, {127, 0, 255}, {255, 0, 255}, {255, 0, 127}
                            };

// Light Control
int lightMode = 0;
bool lightOn = true;

float lightBrightness = 1.0f;
float lightBrightnessStep = 0.1f;

int lightSleep = 250;
int lightSleepStep = 50;
int lightSleepMin = 50;
int lightSleepMax = 1000;

// char buffer for Serial PrintF
char printBuffer[40];

void setup() {

  // Init Serial
  Serial.begin(115200);

  // Setup LED Strip
  FastLED.addLeds<WS2812, LED1_PIN, GRB>(leds1, NUM_LEDS);
  FastLED.addLeds<WS2812, LED2_PIN, GRB>(leds2, NUM_LEDS);

  // Setup IR Remote
  IrReceiver.begin(IR_RECEIVE_PIN, ENABLE_LED_FEEDBACK);
}

void processIrRemote() {
  if (IrReceiver.decode()) {
      // Serial.println(IrReceiver.decodedIRData.decodedRawData, HEX); // Print "old" raw data
      // IrReceiver.printIRResultShort(&Serial); // Print complete received data in one line
      // IrReceiver.printIRSendUsage(&Serial);   // Print the statement required to send this data

      // USE THIS DEBUG INFO TO MAP REMOTE KEYS
      snprintf(printBuffer, sizeof printBuffer, "ir Command:%x\n", IrReceiver.decodedIRData.command);
      Serial.print(printBuffer);

      switch (IrReceiver.decodedIRData.command) {
        case 0x8: // left
          lightSleep -= lightSleepStep;
          if (lightSleep < lightSleepMin) lightSleep = lightSleepMin;
          break;
        case 0x5a: // right
          lightSleep += lightSleepStep;
          if (lightSleep > lightSleepMax) lightSleep = lightSleepMax;
          break;
        case 0x18: // up
          lightBrightness += lightBrightnessStep;
          if (lightBrightness > 1.0f) lightBrightness = 1.0f;
          break;
        case 0x52: // down
          lightBrightness -= lightBrightnessStep;
          if (lightBrightness < 0.0f) lightBrightness = 0.0f;
          break;
        case 0x1c: // Ok
          lightOn = !lightOn;
          break;
        case 0x19:  //  0
          lightMode = 0;
          break;
        case 0x45:  //  1
          lightMode = 1;
          break;
        case 0x46:  //  2
          lightMode = 2;
          break;
        case 0x47:  //  3
          lightMode = 3;
          break;
        case 0x44:  //  4
          lightMode = 4;
          break;
        case 0x40:  //  5
          lightMode = 5;
          break;
        case 0x43:  //  6
          lightMode = 6;
          break;
        case 0x07:  //  7
          lightMode = 7;
          break;
        case 0x15:  //  8
          lightMode = 8;
          break;
        case 0x09:  //  9
          lightMode = 9;
          break;
        case 0x16:  // *
          lightMode--;
          if (lightMode < 0) lightMode = 9;
          break;
        case 0xd:   // #
          lightMode++;
          if (lightMode > 9) lightMode = 0;
            break;
        default:
          break;
      }

      IrReceiver.resume(); // Enable receiving of the next value
  }
}

void updateLight() {

  if (lightOn == false) {
    FastLED.clear();
    FastLED.show();
    return;
  }; 

  for (int i = 0; i < NUM_LEDS; i++) {
    int c = start + i;
    if (c >= NUM_LEDS) c -= NUM_LEDS;
    leds1[i] = getColor(c, lightBrightness);
    leds2[i] = getColor(c, lightBrightness);

    // snprintf(printBuffer, sizeof printBuffer, "i:%d, Angle %d,RGB - %d, %d, %d\n", i, angle + i * step, r, g, b);
    // Serial.print(printBuffer);
  }
  FastLED.show();

  start += 1;
  if (start >= NUM_LEDS) start = 0;

}

CRGB getColor(int index, float brightness) {
  switch (lightMode) {
    case 0:
      return CRGB(rgbArray[index][0] * brightness, rgbArray[index][1] * brightness, rgbArray[index][2] * brightness);   // mode 0 - RGB
    case 1:
      return CRGB(rgbArray[index][0] * brightness, 0, 0);   // mode 1   Red
    case 2:
      return CRGB(0, rgbArray[index][1] * brightness, 0);   // mode 2   Blue
    case 3:
      return CRGB(0, 0, rgbArray[index][2] * brightness);   // mode 3   Green
    case 4:
      return CRGB(rgbArray[index][0] * brightness, rgbArray[index][0] * brightness, rgbArray[index][0] * brightness);   // mode 4   White
    case 5:
    case 6:
    case 7:
    case 8:
    case 9:
    default:
      return CRGB(rgbArray[index][0] * brightness, rgbArray[index][1] * brightness, rgbArray[index][2] * brightness);   // mode 5-9 TODO
  }
}

void loop() {

  processIrRemote();
  updateLight();
  
  delay(lightSleep);
}